mod parser;
mod gui;
mod state;

use glium::Surface;
use imgui::{Condition, Context, FocusedWidget, Ui, WindowFlags};
use lalrpop_util::lalrpop_mod;
use crate::state::State;

lalrpop_mod!(grammar);

fn render(main_pos_size: ([f32; 2], [f32; 2]), state: &mut State, ui: &mut Ui) {
    gui::draw_main_menu_bar(state, ui);

    let wflags = WindowFlags::NO_SCROLLBAR | WindowFlags::NO_MOVE | WindowFlags::NO_DECORATION | WindowFlags::NO_RESIZE | WindowFlags::NO_TITLE_BAR | WindowFlags::NO_COLLAPSE | WindowFlags::NO_DOCKING;
    if let Some(t) = ui.window("MainWindow")
        .position(main_pos_size.0, Condition::Always)
        .size(main_pos_size.1, Condition::Always)
        .flags(wflags)
        .begin() {

        ui.child_window("InputText")
            .size([0.0, 300.0])
            .always_vertical_scrollbar(true)
            .border(true)
            .build(|| state.inputs.iter().for_each(|i| ui.text(i)));

        if let Some(err) = &state.error_msg {
            ui.text_colored([1.0, 0.0, 0.0, 1.0], "Error: ");
            ui.text(err);
        }

        ui.text("Input: ");
        ui.same_line();
        if ui.input_text("##Input", &mut state.input_buf)
            .enter_returns_true(true)
            .build() {
            state.handle_input_buf();
            ui.set_keyboard_focus_here_with_offset(FocusedWidget::Previous);
        }
        if let Some(last_eval) = &state.last_eval {
            ui.same_line();
            ui.text(format!("Eval took: {:?}", last_eval));
        }

        ui.spacing();
        ui.separator();
        ui.spacing();

        if let Some(t) = ui.child_window("Variables").begin() {
            gui::draw_table(state, ui);
            t.end();
        }
        t.end();
    }
}

fn main() {
    let mut state = State::default();
    let ev = glium::glutin::event_loop::EventLoop::new();
    let ctx = glium::glutin::ContextBuilder::new().with_vsync(false);
    let builder = glium::glutin::window::WindowBuilder::new()
        .with_title("MCalc");
    let display = glium::Display::new(builder, ctx, &ev).unwrap();
    let mut imgui = Context::create();
    imgui.set_ini_filename(None);
    let mut platform = imgui_winit_support::WinitPlatform::init(&mut imgui);
    {
        let glw = display.gl_window();
        let window = glw.window();
        platform.attach_window(imgui.io_mut(), window, imgui_winit_support::HiDpiMode::Locked(1.0));
    }
    let mut renderer = imgui_glium_renderer::Renderer::init(&mut imgui, &display).unwrap();
    let mut last_frame = std::time::Instant::now();
    ev.run(move |e, _, flow| match e {
        glium::glutin::event::Event::NewEvents(_) => {
            let now = std::time::Instant::now();
            imgui.io_mut().update_delta_time(now - last_frame);
            last_frame = now;
        }
        glium::glutin::event::Event::MainEventsCleared => {
            let glw = display.gl_window();
            platform.prepare_frame(imgui.io_mut(), glw.window()).unwrap();
            glw.window().request_redraw();
        }
        glium::glutin::event::Event::RedrawRequested(_) => {
            let main_pos_size = {
                let v = imgui.main_viewport();
                (v.work_pos, v.work_size)
            };
            let ui = imgui.new_frame();
            render(main_pos_size, &mut state, ui);
            let glw = display.gl_window();
            let mut target = display.draw();
            target.clear_color(0.0, 0.0, 0.0, 1.0);
            platform.prepare_render(ui, glw.window());
            renderer.render(&mut target, imgui.render()).unwrap();
            target.finish().unwrap();
        }
        glium::glutin::event::Event::WindowEvent { event: glium::glutin::event::WindowEvent::CloseRequested, .. } => {
            *flow = glium::glutin::event_loop::ControlFlow::Exit
        }
        _ => {
            let glw = display.gl_window();
            platform.handle_event(imgui.io_mut(), glw.window(), &e);
        }
    })
}
