use std::collections::HashMap;

#[derive(Debug, Clone, serde::Deserialize, serde::Serialize)]
pub enum Node {
    Num(f64),
    Var(String),
    Neg(Box<Node>),
    Plus(Box<Node>, Box<Node>),
    Minus(Box<Node>, Box<Node>),
    Mul(Box<Node>, Box<Node>),
    Div(Box<Node>, Box<Node>)
}

fn join_vec<T>(mut l: Vec<T>, mut r: Vec<T>) -> Vec<T> {
    l.append(&mut r);
    l
}

impl Node {
    pub fn used_vars(&self) -> Vec<String> {
        match self {
            Node::Num(_) => vec![],
            Node::Var(v) => vec![v.clone()],
            Node::Neg(n) => n.used_vars(),
            Node::Plus(l, r) => join_vec(l.used_vars(), r.used_vars()),
            Node::Minus(l, r) => join_vec(l.used_vars(), r.used_vars()),
            Node::Mul(l, r) => join_vec(l.used_vars(), r.used_vars()),
            Node::Div(l, r) => join_vec(l.used_vars(), r.used_vars())
        }
    }

    pub fn eval(&self, vars: &HashMap<String, Node>) -> Option<f64> {
        match self {
            Node::Num(v) => Some(v.clone()),
            Node::Var(v) => vars.get(v)?.eval(vars),
            Node::Neg(n) => n.eval(vars),
            Node::Plus(l, r) =>
                l.eval(vars).and_then(|l| r.eval(vars).and_then(|r| Some(l + r))),
            Node::Minus(l, r) =>
                l.eval(vars).and_then(|l| r.eval(vars).and_then(|r| Some(l - r))),
            Node::Mul(l, r) =>
                l.eval(vars).and_then(|l| r.eval(vars).and_then(|r| Some(l * r))),
            Node::Div(l, r) =>
                l.eval(vars).and_then(|l| r.eval(vars).and_then(|r| Some(l / r)))
        }
    }
}

