use crate::parser::Node;

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct Variable {
    pub name: String,
    pub expr: String,

    #[serde(skip)]
    pub value: Option<f64>,

    pub parsed_expr: Node,
    pub used: Vec<String>
}

#[derive(Debug, Default)]
pub struct State {
    pub inputs: Vec<String>,
    pub input_buf: String,
    pub error_msg: Option<String>,
    pub variables: Vec<Variable>,
    pub last_eval: Option<std::time::Duration>
}

impl State {
    pub fn eval_vars(&mut self) {
        let start = std::time::Instant::now();

        let var_map = self.variables.iter()
            .map(|v| (v.name.clone(), v.parsed_expr.clone()))
            .collect();

        for v in &mut self.variables {
            let val = v.parsed_expr.eval(&var_map);
            v.value = val;
        }

        self.last_eval = Some(start.elapsed());
    }

    pub fn handle_input_buf(&mut self) {
        let input = std::mem::take(&mut self.input_buf);
        if !self.handle_input(&input) {
            self.input_buf = input;
        }
    }

    pub fn handle_input(&mut self, input: &String) -> bool {
        self.error_msg.take();

        let node: Result<(String, Node), lalrpop_util::ParseError<usize, lalrpop_util::lexer::Token<'_>, &str>>
            = crate::grammar::AssignParser::new().parse(&input);
        match node {
            Ok((name, expr)) => {
                let used_vars = expr.used_vars();
                if used_vars.contains(&name) {
                    self.error_msg = Some("Variable can't depend on itself".to_string());
                    return false;
                }
                let seen = vec![name.clone()];
                if let Some(mut e) = used_vars.iter().filter_map(|v| self.find_dependency_recursive(v, seen.clone()).err()).next() {
                    e.push(name.clone());
                    e.reverse();
                    e.push(name.clone());
                    self.error_msg = Some(format!("Found recursive dependency:\n{}", e.join(" -> ")));
                    return false;
                }
                if let Some(v) = self.variables.iter_mut().filter(|v| v.name == name).next() {
                    v.parsed_expr = expr;
                    v.expr = input.clone();
                    v.used = used_vars;
                } else {
                    self.variables.push(Variable {
                        name,
                        expr: input.clone(),
                        value: None,
                        parsed_expr: expr,
                        used: used_vars
                    });
                }
            }
            Err(e) => {
                self.error_msg = Some(e.to_string());
                return false;
            }
        }

        self.eval_vars();
        self.inputs.push(input.clone());
        true
    }

    fn find_dependency_recursive(&self, name: &String, mut seen: Vec<String>) -> Result<(), Vec<String>> {
        if let Some(var) = self.variables.iter().filter(|v| v.name == *name).next() {
            seen.push(name.clone());
            if var.used.iter().any(|v| seen.contains(v)) {
                return Err(vec![name.clone()]);
            }
            if let Some(mut e) = var.used.iter().filter_map(|v| self.find_dependency_recursive(v, seen.clone()).err()).next() {
                e.push(name.clone());
                return Err(e);
            }
        }
        Ok(())
    }
}
